@@ -1,5 +1,5 @@
 from .misc import setorder, series, new_var, new_con, new_fun, new_ten, set_space_time, factor_pt as factor,simplify_pt as simplify,expand_pt as expand
-from .tensor import D, C
+from .tensor import D, C, determinant as det
 from .core import display
 from .lcivita import LeviCivita
 from .kdelta import KroneckerDelta
\ No newline at end of file
